function [q,S] = dist2q(dist2, type, alpha, c)
%
% Compute the heavy-tailed embedding similarities for given pairwise 
% squared distances
%
% Input:
%   dist2    pairwise squared distances
%   type     type of heavy-tailed embedding similarities (default student1)
%            student1  --   the function used in t-SNE
%            gauss1    --   spherical Gaussian with zero mean and unit
%                           variance
%            p2, pdot5 --   speedup for power family function with alpha=2
%                           and alpha=0.5, respectively
%   alpha,c  parameters used in the power family
%            q = (alpha.*dist2+c)^(-1/alpha);
% Output:
%   q        the heavy-tailed embedding similarities
%   S        the negative score of the similarities
%
% Zhirong Yang, January 4, 2010

if ~exist('c', 'var') || isempty(c)
    c = 1;
end
if ~exist('type', 'var') || isempty(type)
    type = 'student1';
end

switch type
    case 'student1'
        q = 1 ./ ( 1 + dist2 );
        S = q;
    case 'gauss1'
        q = exp(-dist2);
        S = ones(size(Q));
    case 'p2'
        S = 1 ./ ( 2 * dist2 + c );
        q = sqrt(S);
    case 'pdot5'
        S = 1 ./ ( 0.5 * dist2 + c );
        q = S .* S;
    case 'power'
        if c==1
            switch alpha
                case 1
                    [q,S] = dist2q(dist2, 'student1', alpha, c);
                case 0
                    [q,S] = dist2q(dist2, 'gauss1');
                case 2
                    [q,S] = dist2q(dist2, 'p2', alpha, c);
                case 0.5
                    [q,S] = dist2q(dist2, 'pdot5', alpha, c);
                otherwise
                    [q,S] = power_dist2q(dist2, alpha, c);
            end
        else
            [q,S] = power_dist2q(dist2, alpha, c);
        end
    otherwise
        error('not implemented yet!!');
end
n = size(dist2,1);
for i=1:n
    S(i,i) = 0;
    q(i,i) = 0;
end

function [q,S] = power_dist2q(dist2, alpha, c)
S = 1./(alpha.*dist2+c);
q = power(S, 1/alpha);
