function [Y, Ys] = fphssne_p(P, alpha, c, nSeed, max_iter, disp_step, tol, verbose)
%
% Compute visualization map for pairwise similarities input using
% Heavy-tailed Symmetric Stochastic Neighbor Embedding and the fixed-point
% algorithm
% 
% Y = fphssne(P);
%   Compute the visualization map using default settings
%   Input P: n x n
%   Output Y: n x 2
%
% Y = fphssne(P, alpha, c);
%   Using the power family of heavy-tailed embedding functions
%   Default alpha=1, c=1 (Student t-distribution with single freedom)
%
% Y = fphssne(P, alpha, c, nSeed, max_iter, disp_step, tol, verbose);
%   Specifying more details
%   nSeed         random seed for initializing Y (default 0)
%   max_iter      maximum iterations (default 1000)
%   disp_step     iterval of statistics output (default 10) 
%   tol           convergence tolerance of Y (default 1e-4)
%   verbose       displaying verbosity (default true)
%
% [Y,YS] = fphssne(...);
%    Also return the intermediate visualization maps
%    Ys is a cell array of size (max_iter,1)
%
% Zhirong Yang, January 4, 2010

if ~exist('alpha', 'var') || isempty(alpha)
    alpha = 1;
end
if ~exist('c', 'var') || isempty(c)
    c = 1;
end
if ~exist('nSeed', 'var') || isempty(nSeed)
    nSeed = 0;
end
if ~exist('max_iter', 'var') || isempty(max_iter)
    max_iter = 1000;
end
if ~exist('disp_step', 'var') || isempty(disp_step)
    disp_step = 100;
end
if ~exist('tol', 'var') || isempty(tol)
    tol = 1e-4;
end
if ~exist('verbose', 'var') || isempty(verbose)
    verbose = true;
end

n = size(P,1);

randn('seed', nSeed);
Y = 0.0001 * randn(n,2);

if nargout>1
    Ys = cell(max_iter,1);
end

for iter=1:max_iter
    if verbose && mod(iter, disp_step)==0
        fprintf('%d, ', iter);
    end
    
    Y_old = Y;
    
    Ydist2 = distSqrdSelf(Y);
    [q,S] = dist2q(Ydist2, 'power', alpha, c);
    for i=1:n
        q(i,i) = 0;
    end
    
    Q = q ./ sum(sum(q));
    div = sum(sum(P .* log((P + eps) ./ (Q + eps)), 2));
    if verbose && mod(iter, disp_step)==0
        fprintf('div=%.10f, ', div);
    end
    
    A = P .* S;
    B = Q .* S;
    Y = bsxfun(@rdivide, bsxfun(@times, Y, sum(B,2)) + (A-B)*Y, sum(A,2));
    
    if nargout>1
        Ys{iter} = Y;
    end

    diff_Y = norm(Y-Y_old, 'fro') / norm(Y_old,'fro');
    if verbose && mod(iter, disp_step)==0
        fprintf('diff_Y=%.12f, ', diff_Y);
    end
    
    if diff_Y<tol
        if verbose
            fprintf('converged after %d steps.\n', iter);
        end
        break;
    end
    
    if verbose && mod(iter, disp_step)==0
        fprintf('\n');
    end
end